package zero1hd.rhythmbullet.graphics.ui.pages;

import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Slider;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.rhythmbullet.audio.MusicManager;
import zero1hd.rhythmbullet.audio.analyzer.AudioAnalyzer;

public class AnalysisPage extends Page {
	private boolean confirmed;
	private AudioAnalyzer aa;
	private Table table;
	private Label difficultyModLabel, healthModLabel, speedModLabel;
	private Slider difficultyModifierSlider, healthModifierSlider, speedModifierSlider;
	private Label diffModPercentLabel, heltModPercentLabel, speeModPercentLabel;
	private Label progressLabel;
	private TextButton confirmButton;
	public AnalysisPage(Skin skin, AssetManager assets) {
		setTextureBackground(assets.get("gradients.atlas", TextureAtlas.class).findRegion("red-round"));
		table = new Table();
		table.setFillParent(true);
		table.defaults().space(10f);
		addActor(table);
		
		difficultyModLabel = new Label("Difficulty Modifier: ", skin, "sub-font", skin.getColor("default"));
		difficultyModifierSlider = new Slider(1, 3, 0.5f, false, skin);
		diffModPercentLabel = new Label(String.valueOf(difficultyModifierSlider.getValue()) + "x", skin);
		difficultyModifierSlider.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				diffModPercentLabel.setText(String.valueOf(difficultyModifierSlider.getValue()) + "x");
			}
		});
		table.add(difficultyModLabel);
		table.add(difficultyModifierSlider).minWidth(0.5f*getWidth());
		table.add(diffModPercentLabel).minWidth(diffModPercentLabel.getWidth()*1.5f);
		table.row();
		healthModLabel = new Label("Health Modifier: ", skin, "sub-font", skin.getColor("default"));
		healthModifierSlider = new Slider(1f, 3f, 0.5f, false, skin);
		heltModPercentLabel = new Label(String.valueOf(healthModifierSlider.getValue()) + "x", skin);
		healthModifierSlider.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				heltModPercentLabel.setText(String.valueOf(healthModifierSlider.getValue()) + "x");
			}
		});
		table.add(healthModLabel);
		table.add(healthModifierSlider).fillX();
		table.add(heltModPercentLabel).fillX();
		table.row();
		speedModLabel = new Label("Speed Modifier: ", skin, "sub-font", skin.getColor("default"));
		speedModifierSlider = new Slider(1, 3, 0.5f, false, skin);
		speeModPercentLabel = new Label(String.valueOf(speedModifierSlider.getValue()) + "x", skin);
		speedModifierSlider.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				speeModPercentLabel.setText(String.valueOf(speedModifierSlider.getValue()) + "x");
			}
		});
		table.add(speedModLabel);
		table.add(speedModifierSlider).fillX();
		table.add(speeModPercentLabel).fillX();
		table.row();
		
		confirmButton = new TextButton("Confirm", skin);
		confirmButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				confirmed = true;
				confirmButton.setDisabled(true);
			}
		});
		
		table.add(confirmButton).colspan(3).fillX();
		table.row();
		progressLabel = new Label("Loading... ", skin);
		table.add(progressLabel).colspan(2).left().spaceTop(20f);
	}
	
	public void processSong(MusicManager mm) {
		aa = new AudioAnalyzer(mm);
		aa.start();
	}

	@Override
	public void act(float delta) {
		if (aa != null && aa.isDone() && confirmed) {
			//TODO GAMESCREEN!!!
		}
		super.act(delta);
	}
	
	@Override
	public void dispose() {
		aa.dispose();
		super.dispose();
	}
}
